@@ -14,6 +14,7 @@
 #include <cplusplus/FindUsages.h>
 
 #include <cppeditor/cppcodemodelsettings.h>
+#include <cppeditor/cppeditorwidget.h>
 #include <cppeditor/cppfindreferences.h>
 #include <cppeditor/cpptoolsreuse.h>
 
@@ -21,9 +22,10 @@
 #include <languageserverprotocol/lsptypes.h>
 
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
-#include <projectexplorer/session.h>
+#include <projectexplorer/projectmanager.h>
 
 #include <texteditor/basefilefind.h>
 
@@ -45,7 +47,15 @@ namespace ClangCodeModel::Internal {
 
 class ReferencesFileData {
 public:
-    QList<QPair<Range, QString>> rangesAndLineText;
+    struct ItemData {
+        ItemData(const Range &r, const QString &c, const QString &l)
+            : range(r), container(c), lineText(l) {}
+        Range range;
+        QString container;
+        QString lineText;
+    };
+
+    QList<ItemData> itemData;
     QString fileContent;
     ClangdAstNode ast;
 };
@@ -71,7 +81,7 @@ public:
     const Position linkAsPosition;
     const QPointer<SearchResult> search;
     const LinkHandler callback;
-    QList<SearchResultItem> declDefItems;
+    SearchResultItems declDefItems;
     bool openedExtraFileForLink = false;
     bool declHasUsedTag = false;
     bool recursiveCallDetected = false;
@@ -88,8 +98,9 @@ public:
             const SearchResult *search,
             const ReplacementData &replacementData,
             const QString &newSymbolName,
-            const QList<SearchResultItem> &checkedItems,
-            bool preserveCase);
+            const SearchResultItems &checkedItems,
+            bool preserveCase,
+            bool preferLowerCaseFileNames);
     void handleFindUsagesResult(const QList<Location> &locations);
     void finishSearch();
     void reportAllSearchResultsAndFinish();
@@ -140,17 +151,18 @@ ClangdFindReferences::ClangdFindReferences(ClangdClient *client, TextDocument *d
         const auto renameFilesCheckBox = new QCheckBox;
         renameFilesCheckBox->setVisible(false);
         d->search->setAdditionalReplaceWidget(renameFilesCheckBox);
-        const auto renameHandler =
-                [search = d->search](const QString &newSymbolName,
-                       const QList<SearchResultItem> &checkedItems,
-                       bool preserveCase) {
+        const bool preferLowerCase = CppEditor::preferLowerCaseFileNames(client->project());
+        const auto renameHandler = [search = d->search, preferLowerCase](
+                                       const QString &newSymbolName,
+                                       const SearchResultItems &checkedItems,
+                                       bool preserveCase) {
             const auto replacementData = search->userData().value<ReplacementData>();
             Private::handleRenameRequest(search, replacementData, newSymbolName, checkedItems,
-                                         preserveCase);
+                                         preserveCase, preferLowerCase);
         };
         connect(d->search, &SearchResult::replaceButtonClicked, renameHandler);
     }
-    connect(d->search, &SearchResult::activated, [](const SearchResultItem& item) {
+    connect(d->search, &SearchResult::activated, [](const SearchResultItem &item) {
         EditorManager::openEditorAtSearchResult(item);
     });
     if (d->search->isInteractive())
@@ -187,22 +199,23 @@ ClangdFindReferences::ClangdFindReferences(ClangdClient *client, const Link &lin
     d->categorize = true;
     d->search = search;
 
-    if (!client->documentForFilePath(link.targetFilePath)) {
-        QFile f(link.targetFilePath.toString());
-        if (!f.open(QIODevice::ReadOnly)) {
+    const FilePath &targetFilePath = link.targetFilePath;
+    if (!client->documentForFilePath(targetFilePath)) {
+        expected_str<QByteArray> fileContents = targetFilePath.fileContents();
+        if (!fileContents) {
             d->finishSearch();
             return;
         }
-        const QString contents = QString::fromUtf8(f.readAll());
+        const QString contents = QString::fromUtf8(*std::move(fileContents));
         QTextDocument doc(contents);
         QTextCursor cursor(&doc);
         cursor.setPosition(Text::positionInText(&doc, link.targetLine, link.targetColumn + 1));
         cursor.select(QTextCursor::WordUnderCursor);
         d->searchTerm = cursor.selectedText();
-        client->openExtraFile(link.targetFilePath, contents);
+        client->openExtraFile(targetFilePath, contents);
         d->checkUnusedData->openedExtraFileForLink = true;
     }
-    const TextDocumentIdentifier documentId(client->hostPathToServerUri(link.targetFilePath));
+    const TextDocumentIdentifier documentId(client->hostPathToServerUri(targetFilePath));
     const Position pos(link.targetLine - 1, link.targetColumn);
     ReferenceParams params(TextDocumentPositionParams(documentId, pos));
     params.setContext(ReferenceParams::ReferenceContext(true));
@@ -242,15 +255,14 @@ void ClangdFindReferences::Private::handleRenameRequest(
         const SearchResult *search,
         const ReplacementData &replacementData,
         const QString &newSymbolName,
-        const QList<SearchResultItem> &checkedItems,
-        bool preserveCase)
+        const SearchResultItems &checkedItems,
+        bool preserveCase,
+        bool preferLowerCaseFileNames)
 {
     const Utils::FilePaths filePaths = BaseFileFind::replaceAll(newSymbolName, checkedItems,
                                                                 preserveCase);
-    if (!filePaths.isEmpty()) {
-        DocumentManager::notifyFilesChangedInternally(filePaths);
+    if (!filePaths.isEmpty())
         SearchResultWindow::instance()->hide();
-    }
 
     const auto renameFilesCheckBox = qobject_cast<QCheckBox *>(search->additionalReplaceWidget());
     QTC_ASSERT(renameFilesCheckBox, return);
@@ -260,7 +272,7 @@ void ClangdFindReferences::Private::handleRenameRequest(
     ProjectExplorerPlugin::renameFilesForSymbol(
                 replacementData.oldSymbolName, newSymbolName,
                 Utils::toList(replacementData.fileRenameCandidates),
-                CppEditor::preferLowerCaseFileNames());
+                preferLowerCaseFileNames);
 }
 
 void ClangdFindReferences::Private::handleFindUsagesResult(const QList<Location> &locations)
@@ -286,20 +298,27 @@ void ClangdFindReferences::Private::handleFindUsagesResult(const QList<Location>
         finishSearch();
     });
 
-    for (const Location &loc : locations)
-        fileData[loc.uri()].rangesAndLineText.push_back({loc.range(), {}});
+    for (const Location &loc : locations) {
+        fileData[loc.uri()].itemData.emplaceBack(
+            loc.range(), QJsonObject(loc).value("containerName").toString(), QString());
+    }
+    QSet<FilePath> canonicalFilePaths;
     for (auto it = fileData.begin(); it != fileData.end();) {
         const Utils::FilePath filePath = client()->serverUriToHostPath(it.key());
         if (!filePath.exists()) { // https://github.com/clangd/clangd/issues/935
             it = fileData.erase(it);
             continue;
         }
+        if (!Utils::insert(canonicalFilePaths, filePath.canonicalPath())) { // QTCREATORBUG-30546
+            it = fileData.erase(it);
+            continue;
+        }
         const QStringList lines = SymbolSupport::getFileContents(filePath);
         it->fileContent = lines.join('\n');
-        for (auto &rangeWithText : it.value().rangesAndLineText) {
-            const int lineNo = rangeWithText.first.start().line();
+        for (ReferencesFileData::ItemData &itemData : it.value().itemData) {
+            const int lineNo = itemData.range.start().line();
             if (lineNo >= 0 && lineNo < lines.size())
-                rangeWithText.second = lines.at(lineNo);
+                itemData.lineText = lines.at(lineNo);
         }
         ++it;
     }
@@ -390,15 +409,15 @@ static Usage::Tags getUsageType(const ClangdAstPath &path, const QString &search
 void ClangdFindReferences::Private::addSearchResultsForFile(const FilePath &file,
                                                             const ReferencesFileData &fileData)
 {
-    QList<SearchResultItem> items;
+    SearchResultItems items;
     qCDebug(clangdLog) << file << "has valid AST:" << fileData.ast.isValid();
     const auto expectedDeclTypes = [this]() -> QStringList {
         if (checkUnusedData)
             return {"Function", "CXXMethod"};
         return {};
     }();
-    for (const auto &rangeWithText : fileData.rangesAndLineText) {
-        const Range &range = rangeWithText.first;
+    for (const ReferencesFileData::ItemData &itemData : fileData.itemData) {
+        const Range &range = itemData.range;
         const ClangdAstPath astPath = getAstPath(fileData.ast, range);
         const Usage::Tags usageType = fileData.ast.isValid()
                 ? getUsageType(astPath, searchTerm, expectedDeclTypes)
@@ -424,7 +443,7 @@ void ClangdFindReferences::Private::addSearchResultsForFile(const FilePath &file
                 isProperUsage = !isRecursiveCall;
             }
             if (isProperUsage) {
-                qCDebug(clangdLog) << "proper usage at" << rangeWithText.second;
+                qCDebug(clangdLog) << "proper usage at" << itemData.lineText;
                 canceled = true;
                 finishSearch();
                 return;
@@ -436,24 +455,25 @@ void ClangdFindReferences::Private::addSearchResultsForFile(const FilePath &file
         item.setFilePath(file);
         item.setMainRange(SymbolSupport::convertRange(range));
         item.setUseTextEditorFont(true);
-        item.setLineText(rangeWithText.second);
+        item.setLineText(itemData.lineText);
         if (checkUnusedData) {
-            if (rangeWithText.second.contains("template<>")) {
+            if (itemData.lineText.contains("template<>")) {
                 // Hack: Function specializations are not detectable in the AST.
                 canceled = true;
                 finishSearch();
                 return;
             }
-            qCDebug(clangdLog) << "collecting decl/def" << rangeWithText.second;
+            qCDebug(clangdLog) << "collecting decl/def" << itemData.lineText;
             checkUnusedData->declDefItems << item;
             continue;
         }
-        item.setContainingFunctionName(getContainingFunction(astPath, range).detail());
+        item.setContainingFunctionName(itemData.container);
 
         if (search->supportsReplace()) {
-            const bool fileInSession = SessionManager::projectForFile(file);
-            item.setSelectForReplacement(fileInSession);
-            if (fileInSession && file.baseName().compare(replacementData->oldSymbolName,
+            const Node * const node = ProjectTree::nodeForFile(file);
+            item.setSelectForReplacement(!ProjectManager::hasProjects()
+                                         || (node && !node->isGenerated()));
+            if (node && file.baseName().compare(replacementData->oldSymbolName,
                                                          Qt::CaseInsensitive) == 0) {
                 replacementData->fileRenameCandidates << file;
             }
@@ -594,6 +614,19 @@ static Usage::Tags getUsageType(const ClangdAstPath &path, const QString &search
                     tags |= Usage::Tag::Operator;
                 }
             }
+            if (pathIt->kind() == "CXXMethod") {
+                const ClangdAstNode &classNode = *std::next(pathIt);
+                if (classNode.hasChild([&](const ClangdAstNode &n) {
+                    if (n.kind() != "StaticAssert")
+                        return false;
+                    return n.hasChild([&](const ClangdAstNode &n) {
+                        return n.arcanaContains("Q_PROPERTY"); }, true)
+                           && n.hasChild([&](const ClangdAstNode &n) {
+                                  return n.arcanaContains(" " + searchTerm); }, true);
+                    }, false)) {
+                    tags |= Usage::Tag::MocInvokable;
+                }
+            }
             return tags;
         }
         if (pathIt->kind() == "MemberInitializer")
@@ -653,10 +686,10 @@ ClangdFindReferences::CheckUnusedData::~CheckUnusedData()
 class ClangdFindLocalReferences::Private
 {
 public:
-    Private(ClangdFindLocalReferences *q, TextDocument *document, const QTextCursor &cursor,
+    Private(ClangdFindLocalReferences *q, CppEditorWidget *editorWidget, const QTextCursor &cursor,
             const RenameCallback &callback)
-        : q(q), document(document), cursor(cursor), callback(callback),
-          uri(client()->hostPathToServerUri(document->filePath())),
+        : q(q), editorWidget(editorWidget), document(editorWidget->textDocument()), cursor(cursor),
+          callback(callback), uri(client()->hostPathToServerUri(document->filePath())),
           revision(document->document()->revision())
     {}
 
@@ -668,6 +701,7 @@ public:
     void finish();
 
     ClangdFindLocalReferences * const q;
+    const QPointer<CppEditorWidget> editorWidget;
     const QPointer<TextDocument> document;
     const QTextCursor cursor;
     RenameCallback callback;
@@ -677,9 +711,9 @@ public:
 };
 
 ClangdFindLocalReferences::ClangdFindLocalReferences(
-        ClangdClient *client, TextDocument *document, const QTextCursor &cursor,
-        const RenameCallback &callback)
-    : QObject(client), d(new Private(this, document, cursor, callback))
+    ClangdClient *client, CppEditorWidget *editorWidget, const QTextCursor &cursor,
+    const RenameCallback &callback)
+    : QObject(client), d(new Private(this, editorWidget, cursor, callback))
 {
     d->findDefinition();
 }
@@ -695,7 +729,11 @@ void ClangdFindLocalReferences::Private::findDefinition()
         if (sentinel)
             getDefinitionAst(l);
     };
-    client()->symbolSupport().findLinkAt(document, cursor, linkHandler, true);
+    client()->symbolSupport().findLinkAt(document,
+                                         cursor,
+                                         linkHandler,
+                                         true,
+                                         LanguageClient::LinkTarget::SymbolDef);
 }
 
 void ClangdFindLocalReferences::Private::getDefinitionAst(const Link &link)
@@ -763,7 +801,7 @@ void ClangdFindLocalReferences::Private::handleReferences(const QList<Location>
         return loc.toLink(mapper);
     };
 
-    const Utils::Links links = Utils::transform(references, transformLocation);
+    Utils::Links links = Utils::transform(references, transformLocation);
 
     // The callback only uses the symbol length, so we just create a dummy.
     // Note that the calculation will be wrong for identifiers with
@@ -771,7 +809,27 @@ void ClangdFindLocalReferences::Private::handleReferences(const QList<Location>
     QString symbol;
     if (!references.isEmpty()) {
         const Range r = references.first().range();
-        symbol = QString(r.end().character() - r.start().character(), 'x');
+        const Position pos = r.start();
+        symbol = QString(r.end().character() - pos.character(), 'x');
+        if (editorWidget && document) {
+            QTextCursor cursor(document->document());
+            cursor.setPosition(Text::positionInText(document->document(), pos.line() + 1,
+                                                    pos.character() + 1));
+            const QList<Text::Range> occurrencesInComments
+                = symbolOccurrencesInDeclarationComments(editorWidget, cursor);
+            for (const Text::Range &range : occurrencesInComments) {
+                static const auto cmp = [](const Link &l, const Text::Range &r) {
+                    if (l.targetLine < r.begin.line)
+                        return true;
+                    if (l.targetLine > r.begin.line)
+                        return false;
+                    return l.targetColumn < r.begin.column;
+                };
+                const auto it = std::lower_bound(links.begin(), links.end(), range, cmp);
+                links.emplace(it, links.first().targetFilePath, range.begin.line,
+                              range.begin.column);
+            }
+        }
     }
     callback(symbol, links, revision);
     callback = {};
