@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "customwizard.h"
 #include "customwizardparameters.h"
@@ -37,6 +15,7 @@
 
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
+#include <utils/environment.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
 
@@ -48,26 +27,27 @@
 #include <QFileInfo>
 #include <QCoreApplication>
 
-static const char templatePathC[] = "templates/wizards";
-static const char configFileC[] = "wizard.xml";
+using namespace Core;
+using namespace Utils;
 
-namespace {
-bool enableLoadTemplateFiles()
+namespace ProjectExplorer {
+
+const char templatePathC[] = "templates/wizards";
+const char configFileC[] = "wizard.xml";
+
+static bool enableLoadTemplateFiles()
 {
 #ifdef WITH_TESTS
-    static bool value = qEnvironmentVariableIsEmpty("QTC_DISABLE_LOAD_TEMPLATES_FOR_TEST");
+    static bool value = qtcEnvironmentVariableIsEmpty("QTC_DISABLE_LOAD_TEMPLATES_FOR_TEST");
 #else
     static bool value = true;
 #endif
     return value;
 }
-}
-
-namespace ProjectExplorer {
 
 static QList<ICustomWizardMetaFactory *> g_customWizardMetaFactories;
 
-ICustomWizardMetaFactory::ICustomWizardMetaFactory(const QString &klass, Core::IWizardFactory::WizardKind kind) :
+ICustomWizardMetaFactory::ICustomWizardMetaFactory(const QString &klass, IWizardFactory::WizardKind kind) :
     m_klass(klass), m_kind(kind)
 {
     g_customWizardMetaFactories.append(this);
@@ -98,8 +78,8 @@ class CustomWizardPrivate {
 public:
     CustomWizardPrivate() : m_context(new CustomWizardContext) {}
 
-    QSharedPointer<CustomWizardParameters> m_parameters;
-    QSharedPointer<CustomWizardContext> m_context;
+    std::shared_ptr<CustomWizardParameters> m_parameters;
+    std::shared_ptr<CustomWizardContext> m_context;
     static int verbose;
 };
 
@@ -147,8 +127,8 @@ void CustomWizard::setParameters(const CustomWizardParametersPtr &p)
     d->m_parameters = p;
 
     setId(p->id);
-    setSupportedProjectTypes((p->kind == Core::IWizardFactory::FileWizard)
-                             ?  QSet<Core::Id>() : QSet<Core::Id>() << "UNKNOWN_PROJECT");
+    setSupportedProjectTypes((p->kind == IWizardFactory::FileWizard)
+                             ?  QSet<Id>() : QSet<Id>() << "UNKNOWN_PROJECT");
     setIcon(p->icon);
     setDescription(p->description);
     setDisplayName(p->displayName);
@@ -158,19 +138,20 @@ void CustomWizard::setParameters(const CustomWizardParametersPtr &p)
     setFlags(p->flags);
 }
 
-Core::BaseFileWizard *CustomWizard::create(QWidget *parent, const Core::WizardDialogParameters &p) const
+BaseFileWizard *CustomWizard::create(const WizardDialogParameters &p) const
 {
-    QTC_ASSERT(!d->m_parameters.isNull(), return nullptr);
-    auto wizard = new Core::BaseFileWizard(this, p.extraValues(), parent);
+    QTC_ASSERT(d->m_parameters, return nullptr);
+    auto wizard = new BaseFileWizard(this, p.extraValues());
 
     d->m_context->reset();
     auto customPage = new CustomWizardPage(d->m_context, parameters());
-    customPage->setPath(p.defaultPath());
+    customPage->setFilePath(p.defaultPath());
     if (parameters()->firstPageId >= 0)
         wizard->setPage(parameters()->firstPageId, customPage);
     else
         wizard->addPage(customPage);
-    foreach (QWizardPage *ep, wizard->extensionPages())
+    const QList<QWizardPage *> pages = wizard->extensionPages();
+    for (QWizardPage *ep : pages)
         wizard->addPage(ep);
     if (CustomWizardPrivate::verbose)
         qDebug() << "initWizardDialog" << wizard << wizard->pageIds();
@@ -179,12 +160,12 @@ Core::BaseFileWizard *CustomWizard::create(QWidget *parent, const Core::WizardDi
 }
 
 // Read out files and store contents with field contents replaced.
-static inline bool createFile(CustomWizardFile cwFile,
-                              const QString &sourceDirectory,
-                              const QString &targetDirectory,
-                              const CustomProjectWizard::FieldReplacementMap &fm,
-                              Core::GeneratedFiles *files,
-                              QString *errorMessage)
+static bool createFile(CustomWizardFile cwFile,
+                       const QString &sourceDirectory,
+                       const QString &targetDirectory,
+                       const CustomProjectWizard::FieldReplacementMap &fm,
+                       GeneratedFiles *files,
+                       QString *errorMessage)
 {
     const QChar slash =  QLatin1Char('/');
     const QString sourcePath = sourceDirectory + slash + cwFile.source;
@@ -195,29 +176,27 @@ static inline bool createFile(CustomWizardFile cwFile,
         qDebug() << "generating " << targetPath << sourcePath << fm;
 
     // Read contents of source file
-    const QFile::OpenMode openMode
-            = cwFile.binary ? QIODevice::ReadOnly : (QIODevice::ReadOnly|QIODevice::Text);
-    Utils::FileReader reader;
-    if (!reader.fetch(sourcePath, openMode, errorMessage))
+    FileReader reader;
+    if (!reader.fetch(FilePath::fromString(sourcePath), errorMessage))
         return false;
 
-    Core::GeneratedFile generatedFile;
-    generatedFile.setPath(targetPath);
+    GeneratedFile generatedFile;
+    generatedFile.setFilePath(FilePath::fromString(targetPath).cleanPath());
     if (cwFile.binary) {
         // Binary file: Set data.
         generatedFile.setBinary(true);
         generatedFile.setBinaryContents(reader.data());
     } else {
         // Template file: Preprocess.
-        const QString contentsIn = QString::fromLocal8Bit(reader.data());
+        const QString contentsIn = QString::fromLocal8Bit(reader.text());
         generatedFile.setContents(CustomWizardContext::processFile(fm, contentsIn));
     }
 
-    Core::GeneratedFile::Attributes attributes = {};
+    GeneratedFile::Attributes attributes;
     if (cwFile.openEditor)
-        attributes |= Core::GeneratedFile::OpenEditorAttribute;
+        attributes |= GeneratedFile::OpenEditorAttribute;
     if (cwFile.openProject)
-        attributes |= Core::GeneratedFile::OpenProjectAttribute;
+        attributes |= GeneratedFile::OpenProjectAttribute;
     generatedFile.setAttributes(attributes);
     files->push_back(generatedFile);
     return true;
@@ -227,7 +206,8 @@ static inline bool createFile(CustomWizardFile cwFile,
 template <class WizardPage>
         WizardPage *findWizardPage(const QWizard *w)
 {
-    foreach (int pageId, w->pageIds())
+    const QList<int> ids = w->pageIds();
+    for (const int pageId : ids)
         if (auto wp = qobject_cast<WizardPage *>(w->page(pageId)))
             return wp;
     return nullptr;
@@ -235,24 +215,24 @@ template <class WizardPage>
 
 // Determine where to run the generator script. The user may specify
 // an expression subject to field replacement, default is the target path.
-static inline QString scriptWorkingDirectory(const QSharedPointer<CustomWizardContext> &ctx,
-                                             const QSharedPointer<CustomWizardParameters> &p)
+static inline QString scriptWorkingDirectory(const std::shared_ptr<CustomWizardContext> &ctx,
+                                             const std::shared_ptr<CustomWizardParameters> &p)
 {
     if (p->filesGeneratorScriptWorkingDirectory.isEmpty())
-        return ctx->targetPath;
+        return ctx->targetPath.toUrlishString();
     QString path = p->filesGeneratorScriptWorkingDirectory;
     CustomWizardContext::replaceFields(ctx->replacements, &path);
     return path;
 }
 
-Core::GeneratedFiles CustomWizard::generateFiles(const QWizard *dialog, QString *errorMessage) const
+GeneratedFiles CustomWizard::generateFiles(const QWizard *dialog, QString *errorMessage) const
 {
     // Look for the Custom field page to find the path
     const CustomWizardPage *cwp = findWizardPage<CustomWizardPage>(dialog);
-    QTC_ASSERT(cwp, return Core::GeneratedFiles());
+    QTC_ASSERT(cwp, return {});
 
     CustomWizardContextPtr ctx = context();
-    ctx->path = ctx->targetPath = cwp->path();
+    ctx->path = ctx->targetPath = cwp->filePath();
     ctx->replacements = replacementMap(dialog);
     if (CustomWizardPrivate::verbose) {
         QString logText;
@@ -266,9 +246,9 @@ Core::GeneratedFiles CustomWizard::generateFiles(const QWizard *dialog, QString
     return generateWizardFiles(errorMessage);
 }
 
-bool CustomWizard::writeFiles(const Core::GeneratedFiles &files, QString *errorMessage) const
+bool CustomWizard::writeFiles(const GeneratedFiles &files, QString *errorMessage) const
 {
-    if (!Core::BaseFileWizardFactory::writeFiles(files, errorMessage))
+    if (!BaseFileWizardFactory::writeFiles(files, errorMessage))
         return false;
     if (d->m_parameters->filesGeneratorScript.isEmpty())
         return true;
@@ -276,7 +256,7 @@ bool CustomWizard::writeFiles(const Core::GeneratedFiles &files, QString *errorM
     // project wizard that is entirely created by a script,
     // the target project directory might not exist.
     // Known issue: By nature, the script does not honor
-    // Core::GeneratedFile::KeepExistingFileAttribute.
+    // GeneratedFile::KeepExistingFileAttribute.
     const CustomWizardContextPtr ctx = context();
     const QString scriptWorkingDir = scriptWorkingDirectory(ctx, d->m_parameters);
     const QDir scriptWorkingDirDir(scriptWorkingDir);
@@ -295,19 +275,21 @@ bool CustomWizard::writeFiles(const Core::GeneratedFiles &files, QString *errorM
                                                   ctx->replacements, errorMessage))
         return false;
     // Paranoia: Check on the files generated by the script:
-    foreach (const Core::GeneratedFile &generatedFile, files)
-        if (generatedFile.attributes() & Core::GeneratedFile::CustomGeneratorAttribute)
-            if (!QFileInfo(generatedFile.path()).isFile()) {
+    for (const GeneratedFile &generatedFile : files) {
+        if (generatedFile.attributes() & GeneratedFile::CustomGeneratorAttribute)
+            if (!generatedFile.filePath().isFile()) {
                 *errorMessage = QString::fromLatin1("%1 failed to generate %2").
-                        arg(d->m_parameters->filesGeneratorScript.back(), generatedFile.path());
+                        arg(d->m_parameters->filesGeneratorScript.back()).
+                        arg(generatedFile.filePath().toUrlishString());
                 return false;
             }
+    }
     return true;
 }
 
-Core::GeneratedFiles CustomWizard::generateWizardFiles(QString *errorMessage) const
+GeneratedFiles CustomWizard::generateWizardFiles(QString *errorMessage) const
 {
-    Core::GeneratedFiles rc;
+    GeneratedFiles rc;
     const CustomWizardContextPtr ctx = context();
 
     QTC_ASSERT(!ctx->targetPath.isEmpty(), return rc);
@@ -327,9 +309,11 @@ Core::GeneratedFiles CustomWizard::generateWizardFiles(QString *errorMessage) co
             return rc;
     }
     // Add the template files specified by the <file> elements.
-    foreach (const CustomWizardFile &file, d->m_parameters->files)
-        if (!createFile(file, d->m_parameters->directory, ctx->targetPath, context()->replacements, &rc, errorMessage))
-            return Core::GeneratedFiles();
+    for (const CustomWizardFile &file : std::as_const(d->m_parameters->files))
+        if (!createFile(file, d->m_parameters->directory, ctx->targetPath.toUrlishString(), context()->replacements,
+                        &rc, errorMessage))
+            return {};
+
     return rc;
 }
 
@@ -379,17 +363,13 @@ CustomWizard *CustomWizard::createWizard(const CustomProjectWizard::CustomWizard
     containing valid configuration files and parse them into wizards.
 */
 
-QList<Core::IWizardFactory *> CustomWizard::createWizards()
+void CustomWizard::createWizards()
 {
     QString errorMessage;
     QString verboseLog;
-    const QString templateDirName = Core::ICore::resourcePath() +
-                                    QLatin1Char('/') + QLatin1String(templatePathC);
-
-
-    const QString userTemplateDirName = Core::ICore::userResourcePath() +
-                                        QLatin1Char('/') + QLatin1String(templatePathC);
 
+    const QString templateDirName = ICore::resourcePath(templatePathC).toUrlishString();
+    const QString userTemplateDirName = ICore::userResourcePath(templatePathC).toUrlishString();
 
     const QDir templateDir(templateDirName);
     if (CustomWizardPrivate::verbose)
@@ -397,7 +377,7 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
     if (!templateDir.exists()) {
         if (CustomWizardPrivate::verbose)
            qWarning("Custom project template path %s does not exist.", qPrintable(templateDir.absolutePath()));
-        return QList<Core::IWizardFactory *>();
+        return;
     }
 
     const QDir userTemplateDir(userTemplateDirName);
@@ -429,8 +409,9 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
             switch (parameters->parse(dir.absoluteFilePath(configFile), &errorMessage)) {
             case CustomWizardParameters::ParseOk:
                 if (!Utils::contains(toCreate, [parameters](CustomWizardParametersPtr p) { return parameters->id == p->id; })) {
-                    parameters->directory = dir.absolutePath();
                     toCreate.append(parameters);
+                    parameters->directory = dir.absolutePath();
+                    IWizardFactory::registerFactoryCreator([parameters] { return createWizard(parameters); });
                 } else {
                     verboseLog += QString::fromLatin1("Customwizard: Ignoring wizard in %1 due to duplicate Id %2.\n")
                             .arg(dir.absolutePath()).arg(parameters->id.toString());
@@ -456,23 +437,6 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
             }
         }
     }
-
-    QList<Core::IWizardFactory *> rc;
-    foreach (CustomWizardParametersPtr p, toCreate) {
-        if (CustomWizard *w = createWizard(p)) {
-            rc.push_back(w);
-        } else {
-            qWarning("Custom wizard factory function failed for %s from %s.",
-                     qPrintable(p->id.toString()), qPrintable(p->directory));
-        }
-    }
-
-
-    if (CustomWizardPrivate::verbose) { // Print to output pane for Windows.
-        qWarning("%s", qPrintable(verboseLog));
-        Core::MessageManager::write(verboseLog, Core::MessageManager::ModeSwitch);
-    }
-    return rc;
 }
 
 /*!
@@ -495,10 +459,9 @@ CustomProjectWizard::CustomProjectWizard() = default;
     initProjectWizardDialog() needs to be called.
 */
 
-Core::BaseFileWizard *CustomProjectWizard::create(QWidget *parent,
-                                     const Core::WizardDialogParameters &parameters) const
+BaseFileWizard *CustomProjectWizard::create(const WizardDialogParameters &parameters) const
 {
-    auto projectDialog = new BaseProjectWizardDialog(this, parent, parameters);
+    auto projectDialog = new BaseProjectWizardDialog(this, parameters);
     initProjectWizardDialog(projectDialog,
                             parameters.defaultPath(),
                             projectDialog->extensionPages());
@@ -506,11 +469,11 @@ Core::BaseFileWizard *CustomProjectWizard::create(QWidget *parent,
 }
 
 void CustomProjectWizard::initProjectWizardDialog(BaseProjectWizardDialog *w,
-                                                  const QString &defaultPath,
+                                                  const FilePath &defaultPath,
                                                   const QList<QWizardPage *> &extensionPages) const
 {
     const CustomWizardParametersPtr pa = parameters();
-    QTC_ASSERT(!pa.isNull(), return);
+    QTC_ASSERT(pa, return);
 
     const CustomWizardContextPtr ctx = context();
     ctx->reset();
@@ -524,32 +487,32 @@ void CustomProjectWizard::initProjectWizardDialog(BaseProjectWizardDialog *w,
         else
             w->addPage(new CustomWizardFieldPage(ctx, pa));
     }
-    foreach (QWizardPage *ep, extensionPages)
+    for (QWizardPage *ep : extensionPages)
         w->addPage(ep);
-    w->setPath(defaultPath);
+    w->setFilePath(defaultPath);
     w->setProjectName(BaseProjectWizardDialog::uniqueProjectName(defaultPath));
 
     connect(w, &BaseProjectWizardDialog::projectParametersChanged,
-            this, &CustomProjectWizard::projectParametersChanged);
+            this, &CustomProjectWizard::handleProjectParametersChanged);
 
     if (CustomWizardPrivate::verbose)
         qDebug() << "initProjectWizardDialog" << w << w->pageIds();
 }
 
-Core::GeneratedFiles CustomProjectWizard::generateFiles(const QWizard *w, QString *errorMessage) const
+GeneratedFiles CustomProjectWizard::generateFiles(const QWizard *w, QString *errorMessage) const
 {
     const auto *dialog = qobject_cast<const BaseProjectWizardDialog *>(w);
-    QTC_ASSERT(dialog, return Core::GeneratedFiles());
+    QTC_ASSERT(dialog, return {});
     // Add project name as macro. Path is here under project directory
     CustomWizardContextPtr ctx = context();
-    ctx->path = dialog->path();
-    ctx->targetPath = ctx->path + QLatin1Char('/') + dialog->projectName();
+    ctx->path = dialog->filePath();
+    ctx->targetPath = ctx->path.pathAppended(dialog->projectName());
     FieldReplacementMap fieldReplacementMap = replacementMap(dialog);
     fieldReplacementMap.insert(QLatin1String("ProjectName"), dialog->projectName());
     ctx->replacements = fieldReplacementMap;
     if (CustomWizardPrivate::verbose)
         qDebug() << "CustomProjectWizard::generateFiles" << dialog << ctx->targetPath << ctx->replacements;
-    const Core::GeneratedFiles generatedFiles = generateWizardFiles(errorMessage);
+    const GeneratedFiles generatedFiles = generateWizardFiles(errorMessage);
     return generatedFiles;
 }
 
@@ -558,13 +521,12 @@ Core::GeneratedFiles CustomProjectWizard::generateFiles(const QWizard *w, QStrin
     the respective attributes set.
 */
 
-bool CustomProjectWizard::postGenerateOpen(const Core::GeneratedFiles &l, QString *errorMessage)
+bool CustomProjectWizard::postGenerateOpen(const GeneratedFiles &l, QString *errorMessage)
 {
     // Post-Generate: Open the project and the editors as desired
-    foreach (const Core::GeneratedFile &file, l) {
-        if (file.attributes() & Core::GeneratedFile::OpenProjectAttribute) {
-            ProjectExplorerPlugin::OpenProjectResult result
-                    = ProjectExplorerPlugin::openProject(file.path());
+    for (const GeneratedFile &file : l) {
+        if (file.attributes() & GeneratedFile::OpenProjectAttribute) {
+            OpenProjectResult result = ProjectExplorerPlugin::openProject(file.filePath());
             if (!result) {
                 if (errorMessage)
                     *errorMessage = result.errorMessage();
@@ -575,19 +537,20 @@ bool CustomProjectWizard::postGenerateOpen(const Core::GeneratedFiles &l, QStrin
     return BaseFileWizardFactory::postGenerateOpenEditors(l, errorMessage);
 }
 
-bool CustomProjectWizard::postGenerateFiles(const QWizard *, const Core::GeneratedFiles &l, QString *errorMessage) const
+bool CustomProjectWizard::postGenerateFiles(const QWizard *, const GeneratedFiles &l, QString *errorMessage) const
 {
     if (CustomWizardPrivate::verbose)
         qDebug() << "CustomProjectWizard::postGenerateFiles()";
     return CustomProjectWizard::postGenerateOpen(l, errorMessage);
 }
 
-void CustomProjectWizard::projectParametersChanged(const QString &project, const QString & path)
+void CustomProjectWizard::handleProjectParametersChanged(const QString &name,
+                                                         const Utils::FilePath &path)
 {
     // Make '%ProjectName%' available in base replacements.
-    context()->baseReplacements.insert(QLatin1String("ProjectName"), project);
+    context()->baseReplacements.insert(QLatin1String("ProjectName"), name);
 
-    emit projectLocationChanged(path + QLatin1Char('/') + project);
+    emit projectLocationChanged(path / name);
 }
 
 } // namespace ProjectExplorer
