@@ -5,43 +5,50 @@
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
+#include <externaldependenciesinterface.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/target.h>
 #include <projectstorage/filestatuscache.h>
 #include <projectstorage/filesystem.h>
-#include <projectstorage/nonlockingmutex.h>
 #include <projectstorage/projectstorage.h>
+#include <projectstorage/projectstorageerrornotifier.h>
 #include <projectstorage/projectstoragepathwatcher.h>
 #include <projectstorage/projectstorageupdater.h>
 #include <projectstorage/qmldocumentparser.h>
 #include <projectstorage/qmltypesparser.h>
-#include <projectstorage/sourcepathcache.h>
-#include <sqlitedatabase.h>
 #include <qmlprojectmanager/qmlproject.h>
 #include <qtsupport/baseqtversion.h>
-#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qtkitaspect.h>
+#include <sourcepathstorage/nonlockingmutex.h>
+#include <sourcepathstorage/sourcepathcache.h>
+#include <sqlitedatabase.h>
 
 #include <asynchronousexplicitimagecache.h>
 #include <asynchronousimagecache.h>
 #include <imagecache/asynchronousimagefactory.h>
 #include <imagecache/explicitimagecacheimageprovider.h>
-#include <imagecache/imagecachecollector.h>
-#include <imagecache/imagecacheconnectionmanager.h>
 #include <imagecache/imagecachedispatchcollector.h>
 #include <imagecache/imagecachegenerator.h>
 #include <imagecache/imagecachestorage.h>
-#include <imagecache/meshimagecachecollector.h>
-#include <imagecache/textureimagecachecollector.h>
 #include <imagecache/timestampprovider.h>
+#include <imagecachecollectors/imagecachecollector.h>
+#include <imagecachecollectors/imagecacheconnectionmanager.h>
+#include <imagecachecollectors/meshimagecachecollector.h>
+#include <imagecachecollectors/textureimagecachecollector.h>
 
-#include <utils/asset.h>
+#include <qmldesignerutils/asset.h>
 
 #include <coreplugin/icore.h>
 
 #include <QDirIterator>
 #include <QFileSystemWatcher>
+#include <QLibraryInfo>
 #include <QQmlEngine>
+#include <QStandardPaths>
+
+using namespace std::chrono;
+using namespace std::chrono_literals;
 
 namespace QmlDesigner {
 
@@ -57,12 +64,12 @@ ProjectExplorer::Target *activeTarget(ProjectExplorer::Project *project)
 
 QString previewDefaultImagePath()
 {
-    return Core::ICore::resourcePath("qmldesigner/welcomepage/images/newThumbnail.png").toString();
+    return Core::ICore::resourcePath("qmldesigner/welcomepage/images/newThumbnail.png").toUrlishString();
 }
 
 QString previewBrokenImagePath()
 {
-    return Core::ICore::resourcePath("qmldesigner/welcomepage/images/noPreview.png").toString();
+    return Core::ICore::resourcePath("qmldesigner/welcomepage/images/noPreview.png").toUrlishString();
 }
 
 ::QmlProjectManager::QmlBuildSystem *getQmlBuildSystem(::ProjectExplorer::Target *target)
@@ -75,15 +82,12 @@ class PreviewTimeStampProvider : public TimeStampProviderInterface
 public:
     Sqlite::TimeStamp timeStamp(Utils::SmallStringView) const override
     {
-        auto now = std::chrono::steady_clock::now();
-
-        return std::chrono::duration_cast<std::chrono::seconds>(now.time_since_epoch()).count();
+        return duration_cast<seconds>(steady_clock::now().time_since_epoch()).count();
     }
 
     Sqlite::TimeStamp pause() const override
     {
-        using namespace std::chrono_literals;
-        return std::chrono::duration_cast<std::chrono::seconds>(1h).count();
+        return duration_cast<seconds>(1h).count();
     }
 };
 
@@ -124,7 +128,7 @@ public:
 
 public:
     Sqlite::Database database{Utils::PathString{
-                                  Core::ICore::cacheResourcePath("imagecache-v2.db").toString()},
+                                  Core::ICore::cacheResourcePath("imagecache-v2.db").toUrlishString()},
                               Sqlite::JournalMode::Wal,
                               Sqlite::LockingMode::Normal};
     ImageCacheStorage<Sqlite::Database> storage{database};
@@ -158,7 +162,7 @@ public:
 
 public:
     Sqlite::Database database{Utils::PathString{
-                                  Core::ICore::cacheResourcePath("previewcache.db").toString()},
+                                  Core::ICore::cacheResourcePath("previewcache.db").toUrlishString()},
                               Sqlite::JournalMode::Wal,
                               Sqlite::LockingMode::Normal};
     ImageCacheStorage<Sqlite::Database> storage{database};
@@ -174,39 +178,60 @@ namespace {
 class ProjectStorageData
 {
 public:
-    ProjectStorageData(::ProjectExplorer::Project *project)
-        : database{project->projectDirectory().pathAppended("projectstorage.db").toString()}
+    ProjectStorageData(::ProjectExplorer::Project *project, PathCacheType &pathCache)
+        : database{project->projectDirectory().pathAppended("projectstorage.db").toUrlishString()}
+        , errorNotifier{pathCache}
+        , fileSystem{pathCache}
+        , qmlDocumentParser{storage, pathCache}
+        , pathWatcher{pathCache, fileSystem, &updater}
         , projectPartId{ProjectPartId::create(
-              pathCache.sourceId(SourcePath{project->projectDirectory().toString() + "/."}).internalId())}
+              pathCache.sourceId(SourcePath{project->projectDirectory().toUrlishString() + "/."}).internalId())}
+        , updater{fileSystem,
+                  storage,
+                  fileStatusCache,
+                  pathCache,
+                  qmlDocumentParser,
+                  qmlTypesParser,
+                  pathWatcher,
+                  errorNotifier,
+                  projectPartId}
     {}
     Sqlite::Database database;
-    ProjectStorage<Sqlite::Database> storage{database, database.isInitialized()};
-    ProjectStorageUpdater::PathCache pathCache{storage};
-    FileSystem fileSystem{pathCache};
+    ProjectStorageErrorNotifier errorNotifier;
+    ProjectStorage storage{database, errorNotifier, database.isInitialized()};
+    FileSystem fileSystem;
     FileStatusCache fileStatusCache{fileSystem};
-    QmlDocumentParser qmlDocumentParser{storage, pathCache};
+    QmlDocumentParser qmlDocumentParser;
     QmlTypesParser qmlTypesParser{storage};
-    ProjectStoragePathWatcher<QFileSystemWatcher, QTimer, ProjectStorageUpdater::PathCache>
-        pathWatcher{pathCache, fileSystem, &updater};
+    ProjectStoragePathWatcher<QFileSystemWatcher, QTimer, PathCacheType> pathWatcher;
     ProjectPartId projectPartId;
-    ProjectStorageUpdater updater{fileSystem,
-                                  storage,
-                                  fileStatusCache,
-                                  pathCache,
-                                  qmlDocumentParser,
-                                  qmlTypesParser,
-                                  pathWatcher,
-                                  projectPartId};
+    ProjectStorageUpdater updater;
 };
 
-std::unique_ptr<ProjectStorageData> createProjectStorageData(::ProjectExplorer::Project *project)
+std::unique_ptr<ProjectStorageData> createProjectStorageData(::ProjectExplorer::Project *project,
+                                                             PathCacheType &pathCache)
 {
     if constexpr (useProjectStorage()) {
-        return std::make_unique<ProjectStorageData>(project);
+        return std::make_unique<ProjectStorageData>(project, pathCache);
     } else {
         return {};
     }
 }
+
+Utils::PathString createDatabasePath(std::string_view name)
+{
+    auto directory = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
+
+    QDir{}.mkpath(directory);
+
+    Utils::PathString path = directory;
+
+    path.append('/');
+    path.append(name);
+
+    return path;
+}
+
 } // namespace
 
 class QmlDesignerProjectManager::QmlDesignerProjectManagerProjectData
@@ -214,6 +239,7 @@ class QmlDesignerProjectManager::QmlDesignerProjectManagerProjectData
 public:
     QmlDesignerProjectManagerProjectData(ImageCacheStorage<Sqlite::Database> &storage,
                                          ::ProjectExplorer::Project *project,
+                                         PathCacheType &pathCache,
                                          ExternalDependenciesInterface &externalDependencies)
         : collector{connectionManager,
                     QSize{300, 300},
@@ -221,7 +247,7 @@ public:
                     externalDependencies,
                     ImageCacheCollectorNullImageHandling::CaptureNullImage}
         , factory{storage, timeStampProvider, collector}
-        , projectStorageData{createProjectStorageData(project)}
+        , projectStorageData{createProjectStorageData(project, pathCache)}
     {}
 
     ImageCacheConnectionManager connectionManager;
@@ -232,35 +258,51 @@ public:
     QPointer<::ProjectExplorer::Target> activeTarget;
 };
 
+class QmlDesignerProjectManager::Data
+{
+public:
+    Sqlite::Database sourcePathDatabase{createDatabasePath("source_path.db"),
+                                        Sqlite::JournalMode::Wal,
+                                        Sqlite::LockingMode::Normal};
+    QmlDesigner::SourcePathStorage sourcePathStorage{sourcePathDatabase,
+                                                     sourcePathDatabase.isInitialized()};
+    PathCache pathCache{sourcePathStorage};
+};
+
 QmlDesignerProjectManager::QmlDesignerProjectManager(ExternalDependenciesInterface &externalDependencies)
-    : m_previewImageCacheData{std::make_unique<PreviewImageCacheData>(externalDependencies)}
+    : m_data{std::make_unique<Data>()}
+    , m_previewImageCacheData{std::make_unique<PreviewImageCacheData>(externalDependencies)}
     , m_externalDependencies{externalDependencies}
 {
     auto editorManager = ::Core::EditorManager::instance();
-    QObject::connect(editorManager, &::Core::EditorManager::editorOpened, [&](auto *editor) {
+    QObject::connect(editorManager, &::Core::EditorManager::editorOpened, &dummy, [&](auto *editor) {
         editorOpened(editor);
     });
-    QObject::connect(editorManager, &::Core::EditorManager::currentEditorChanged, [&](auto *editor) {
-        currentEditorChanged(editor);
-    });
-    QObject::connect(editorManager, &::Core::EditorManager::editorsClosed, [&](const auto &editors) {
-        editorsClosed(editors);
-    });
+    QObject::connect(editorManager,
+                     &::Core::EditorManager::currentEditorChanged,
+                     &dummy,
+                     [&](auto *editor) { currentEditorChanged(editor); });
+    QObject::connect(editorManager,
+                     &::Core::EditorManager::editorsClosed,
+                     &dummy,
+                     [&](const auto &editors) { editorsClosed(editors); });
     auto sessionManager = ::ProjectExplorer::ProjectManager::instance();
     QObject::connect(sessionManager,
                      &::ProjectExplorer::ProjectManager::projectAdded,
+                     &dummy,
                      [&](auto *project) { projectAdded(project); });
     QObject::connect(sessionManager,
                      &::ProjectExplorer::ProjectManager::aboutToRemoveProject,
+                     &dummy,
                      [&](auto *project) { aboutToRemoveProject(project); });
     QObject::connect(sessionManager,
                      &::ProjectExplorer::ProjectManager::projectRemoved,
+                     &dummy,
                      [&](auto *project) { projectRemoved(project); });
 
-    QObject::connect(&m_previewImageCacheData->timer,
-                     &QTimer::timeout,
-                     this,
-                     &QmlDesignerProjectManager::generatePreview);
+    QObject::connect(&m_previewImageCacheData->timer, &QTimer::timeout, &dummy, [&]() {
+        generatePreview();
+    });
 }
 
 QmlDesignerProjectManager::~QmlDesignerProjectManager() = default;
@@ -281,19 +323,24 @@ AsynchronousImageCache &QmlDesignerProjectManager::asynchronousImageCache()
 }
 
 namespace {
-ProjectStorage<Sqlite::Database> *dummyProjectStorage()
+[[maybe_unused]] ProjectStorageType *dummyProjectStorage()
+{
+    return nullptr;
+}
+
+[[maybe_unused]] PathCacheType *dummyPathCache()
 {
     return nullptr;
 }
 
 } // namespace
 
-ProjectStorage<Sqlite::Database> &QmlDesignerProjectManager::projectStorage()
+ProjectStorageDependencies QmlDesignerProjectManager::projectStorageDependencies()
 {
     if constexpr (useProjectStorage()) {
-        return m_projectData->projectStorageData->storage;
+        return {m_projectData->projectStorageData->storage, m_data->pathCache};
     } else {
-        return *dummyProjectStorage();
+        return {*dummyProjectStorage(), *dummyPathCache()};
     }
 }
 
@@ -301,7 +348,6 @@ void QmlDesignerProjectManager::editorOpened(::Core::IEditor *) {}
 
 void QmlDesignerProjectManager::currentEditorChanged(::Core::IEditor *)
 {
-    using namespace std::chrono_literals;
     m_previewImageCacheData->timer.start(10s);
 }
 
@@ -309,84 +355,45 @@ void QmlDesignerProjectManager::editorsClosed(const QList<::Core::IEditor *> &)
 
 namespace {
 
-QtSupport::QtVersion *getQtVersion(::ProjectExplorer::Target *target)
-{
-    if (target)
-        return QtSupport::QtKitAspect::qtVersion(target->kit());
-
-    return {};
-}
-
-[[maybe_unused]] QtSupport::QtVersion *getQtVersion(::ProjectExplorer::Project *project)
-{
-    return getQtVersion(project->activeTarget());
-}
-
-Utils::FilePath qmlPath(::ProjectExplorer::Target *target)
+QString qmlPath(::ProjectExplorer::Target *target)
 {
     auto qt = QtSupport::QtKitAspect::qtVersion(target->kit());
     if (qt)
-        return qt->qmlPath();
+        return qt->qmlPath().path();
 
-    return {};
+    return QLibraryInfo::path(QLibraryInfo::QmlImportsPath);
 }
 
-void projectQmldirPaths(::ProjectExplorer::Target *target, QStringList &qmldirPaths)
+[[maybe_unused]] void projectQmldirPaths(::ProjectExplorer::Target *target, QStringList &qmldirPaths)
 {
     ::QmlProjectManager::QmlBuildSystem *buildSystem = getQmlBuildSystem(target);
 
     const Utils::FilePath projectDirectoryPath = buildSystem->canonicalProjectDir();
-    const QStringList importPaths = buildSystem->importPaths();
-    const QDir projectDirectory(projectDirectoryPath.toString());
 
-    for (const QString &importPath : importPaths)
-        qmldirPaths.push_back(QDir::cleanPath(projectDirectory.absoluteFilePath(importPath))
-                              + "/qmldir");
+    qmldirPaths.push_back(projectDirectoryPath.path());
 }
 
-#ifdef QDS_HAS_QMLPRIVATE
-QStringView currentDirectoryName(const QString &path)
+[[maybe_unused]] void qtQmldirPaths(::ProjectExplorer::Target *target, QStringList &qmldirPaths)
 {
-    auto found = std::find(path.rbegin(), path.rend(), u'/');
-
-    if (found == path.rend())
-        return {};
-
-    return QStringView{found.base(), path.end()};
-}
-bool skipPath(const QString &path)
-{
-    auto directory = currentDirectoryName(path);
-
-    bool skip = directory == u"QtApplicationManager" || directory == u"QtInterfaceFramework"
-                || directory == u"QtOpcUa" || directory == u"Qt3D" || directory == u"Qt3D"
-                || directory == u"Scene2D" || directory == u"Scene3D" || directory == u"QtWayland"
-                || directory == u"Qt5Compat";
-
-    return skip;
-}
-#endif
-
-void qtQmldirPaths([[maybe_unused]] ::ProjectExplorer::Target *target,
-                   [[maybe_unused]] QStringList &qmldirPaths)
-{
-#ifdef QDS_HAS_QMLPRIVATE
-    if (useProjectStorage()) {
-        const QString installDirectory = qmlPath(target).toString();
-        QDirIterator dirIterator{installDirectory, QDir::Dirs, QDirIterator::Subdirectories};
-
-        while (dirIterator.hasNext()) {
-            auto directoryPath = dirIterator.next();
-
-            QString qmldirPath = directoryPath + "/qmldir";
-            if (!skipPath(directoryPath) && QFileInfo::exists(qmldirPath))
-                qmldirPaths.push_back(directoryPath);
-        }
+    if constexpr (useProjectStorage()) {
+        auto qmlRootPath = qmlPath(target);
+        qmldirPaths.push_back(qmlRootPath + "/QtQml");
+        qmldirPaths.push_back(qmlRootPath + "/QtQuick");
+        qmldirPaths.push_back(qmlRootPath + "/QtQuick3D");
+        qmldirPaths.push_back(qmlRootPath + "/Qt5Compat");
     }
-#endif
 }
 
-QStringList qmlDirs(::ProjectExplorer::Target *target)
+[[maybe_unused]] void qtQmldirPathsForLiteDesigner(QStringList &qmldirPaths)
+{
+    if constexpr (useProjectStorage()) {
+        auto qmlRootPath = QLibraryInfo::path(QLibraryInfo::QmlImportsPath);
+        qmldirPaths.push_back(qmlRootPath + "/QtQml");
+        qmldirPaths.push_back(qmlRootPath + "/QtQuick");
+    }
+}
+
+[[maybe_unused]] QStringList directories(::ProjectExplorer::Target *target)
 {
     if (!target)
         return {};
@@ -403,7 +410,20 @@ QStringList qmlDirs(::ProjectExplorer::Target *target)
     return qmldirPaths;
 }
 
-QStringList qmlTypes(::ProjectExplorer::Target *target)
+[[maybe_unused]] QStringList directoriesForLiteDesigner()
+{
+    QStringList qmldirPaths;
+    qmldirPaths.reserve(100);
+
+    qtQmldirPathsForLiteDesigner(qmldirPaths);
+
+    std::sort(qmldirPaths.begin(), qmldirPaths.end());
+    qmldirPaths.erase(std::unique(qmldirPaths.begin(), qmldirPaths.end()), qmldirPaths.end());
+
+    return qmldirPaths;
+}
+
+[[maybe_unused]] QStringList qmlTypes(::ProjectExplorer::Target *target)
 {
     if (!target)
         return {};
@@ -411,24 +431,54 @@ QStringList qmlTypes(::ProjectExplorer::Target *target)
     QStringList qmldirPaths;
     qmldirPaths.reserve(2);
 
-    const QString installDirectory = qmlPath(target).toString();
+    const QString qmlRootPath = qmlPath(target);
 
-    qmldirPaths.append(installDirectory + "/builtins.qmltypes");
-    qmldirPaths.append(installDirectory + "/jsroot.qmltypes");
+    qmldirPaths.append(qmlRootPath + "/builtins.qmltypes");
+    qmldirPaths.append(qmlRootPath + "/jsroot.qmltypes");
 
     qmldirPaths.append(
-        Core::ICore::resourcePath("qmldesigner/projectstorage/fake.qmltypes").toString());
+        Core::ICore::resourcePath("qmldesigner/projectstorage/fake.qmltypes").toUrlishString());
 
     return qmldirPaths;
 }
 
+[[maybe_unused]] QStringList qmlTypesForLiteDesigner()
+{
+    QStringList qmldirPaths;
+    qmldirPaths.reserve(2);
+
+    const auto qmlRootPath = QLibraryInfo::path(QLibraryInfo::QmlImportsPath);
+
+    qmldirPaths.append(qmlRootPath + "/builtins.qmltypes");
+    qmldirPaths.append(qmlRootPath + "/jsroot.qmltypes");
+
+    qmldirPaths.append(
+        Core::ICore::resourcePath("qmldesigner/projectstorage/fake.qmltypes").toUrlishString());
+
+    return qmldirPaths;
+}
+
+QString propertyEditorResourcesPath()
+{
+#ifdef SHARE_QML_PATH
+    if (qEnvironmentVariableIsSet("LOAD_QML_FROM_SOURCE")) {
+        return QLatin1String(SHARE_QML_PATH) + "/propertyEditorQmlSources";
+    }
+#endif
+    return Core::ICore::resourcePath("qmldesigner/propertyEditorQmlSources").toUrlishString();
+}
+
+QString qtCreatorItemLibraryPath()
+{
+    return Core::ICore::resourcePath("qmldesigner/itemLibrary").toUrlishString();
+}
+
 } // namespace
 
 void QmlDesignerProjectManager::projectAdded(::ProjectExplorer::Project *project)
 {
-    m_projectData = std::make_unique<QmlDesignerProjectManagerProjectData>(m_previewImageCacheData->storage,
-                                                                           project,
-                                                                           m_externalDependencies);
+    m_projectData = std::make_unique<QmlDesignerProjectManagerProjectData>(
+        m_previewImageCacheData->storage, project, m_data->pathCache, m_externalDependencies);
     m_projectData->activeTarget = project->activeTarget();
 
     QObject::connect(project, &::ProjectExplorer::Project::fileListChanged, [&]() {
@@ -467,13 +517,13 @@ void QmlDesignerProjectManager::generatePreview()
 
     if (qmlBuildSystem) {
         m_previewImageCacheData->collector.setTarget(m_projectData->activeTarget);
-        m_previewImageCacheData->factory.generate(qmlBuildSystem->mainFilePath().toString().toUtf8());
+        m_previewImageCacheData->factory.generate(qmlBuildSystem->mainFilePath().toUrlishString().toUtf8());
     }
 }
 
 QmlDesignerProjectManager::ImageCacheData *QmlDesignerProjectManager::imageCacheData()
 {
-    std::call_once(imageCacheFlag, [this]() {
+    std::call_once(imageCacheFlag, [this] {
         m_imageCacheData = std::make_unique<ImageCacheData>(m_externalDependencies);
         auto setTargetInImageCache =
             [imageCacheData = m_imageCacheData.get()](ProjectExplorer::Target *target) {
@@ -494,12 +544,12 @@ QmlDesignerProjectManager::ImageCacheData *QmlDesignerProjectManager::imageCache
             m_imageCacheData->nodeInstanceCollector.setTarget(project->activeTarget());
             QObject::connect(project,
                              &ProjectExplorer::Project::activeTargetChanged,
-                             this,
+                             &dummy,
                              setTargetInImageCache);
         }
         QObject::connect(ProjectExplorer::ProjectManager::instance(),
                          &ProjectExplorer::ProjectManager::startupProjectChanged,
-                         this,
+                         &dummy,
                          [=](ProjectExplorer::Project *project) {
                              setTargetInImageCache(activeTarget(project));
                          });
@@ -555,8 +605,20 @@ void QmlDesignerProjectManager::update()
     if (!m_projectData || !m_projectData->projectStorageData)
         return;
 
-    m_projectData->projectStorageData->updater.update(qmlDirs(m_projectData->activeTarget),
-                                                      qmlTypes(m_projectData->activeTarget));
+    try {
+        if constexpr (isUsingQmlDesignerLite()) {
+            m_projectData->projectStorageData->updater.update({directoriesForLiteDesigner(),
+                                                               qmlTypesForLiteDesigner(),
+                                                               propertyEditorResourcesPath(),
+                                                               {qtCreatorItemLibraryPath()}});
+        } else {
+            m_projectData->projectStorageData->updater.update({directories(m_projectData->activeTarget),
+                                                               qmlTypes(m_projectData->activeTarget),
+                                                               propertyEditorResourcesPath(),
+                                                               {qtCreatorItemLibraryPath()}});
+        }
+    } catch (const std::exception &) {
+    }
 }
 
 } // namespace QmlDesigner
