@@ -1,29 +1,8 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "debugmessagesmodel_test.h"
+#include "../qmlprofilertr.h"
 
 #include <tracing/timelineformattime.h>
 
@@ -71,11 +50,11 @@ void DebugMessagesModelTest::testColor()
 }
 
 static const char *messageTypes[] = {
-    QT_TRANSLATE_NOOP("DebugMessagesModel", "Debug Message"),
-    QT_TRANSLATE_NOOP("DebugMessagesModel", "Warning Message"),
-    QT_TRANSLATE_NOOP("DebugMessagesModel", "Critical Message"),
-    QT_TRANSLATE_NOOP("DebugMessagesModel", "Fatal Message"),
-    QT_TRANSLATE_NOOP("DebugMessagesModel", "Info Message"),
+    QT_TRANSLATE_NOOP("QtC::QmlProfiler", "Debug Message"),
+    QT_TRANSLATE_NOOP("QtC::QmlProfiler", "Warning Message"),
+    QT_TRANSLATE_NOOP("QtC::QmlProfiler", "Critical Message"),
+    QT_TRANSLATE_NOOP("QtC::QmlProfiler", "Fatal Message"),
+    QT_TRANSLATE_NOOP("QtC::QmlProfiler", "Info Message"),
 };
 
 void DebugMessagesModelTest::testLabels()
@@ -83,7 +62,7 @@ void DebugMessagesModelTest::testLabels()
     QVariantList labels = model.labels();
     for (int i = 0; i <= QtMsgType::QtInfoMsg; ++i) {
         QVariantMap element = labels[i].toMap();
-        QCOMPARE(element[QLatin1String("description")].toString(), model.tr(messageTypes[i]));
+        QCOMPARE(element[QLatin1String("description")].toString(), Tr::tr(messageTypes[i]));
         QCOMPARE(element[QLatin1String("id")].toInt(), i);
     }
 }
@@ -93,12 +72,12 @@ void DebugMessagesModelTest::testDetails()
     for (int i = 0; i < 10; ++i) {
         QVariantMap details = model.details(i);
         QCOMPARE(details.value(QLatin1String("displayName")).toString(),
-                 model.tr(messageTypes[i % (QtMsgType::QtInfoMsg + 1)]));
-        QCOMPARE(details.value(model.tr("Timestamp")).toString(),
+                 Tr::tr(messageTypes[i % (QtMsgType::QtInfoMsg + 1)]));
+        QCOMPARE(details.value(Tr::tr("Timestamp")).toString(),
                  Timeline::formatTime(i));
-        QCOMPARE(details.value(model.tr("Message")).toString(),
+        QCOMPARE(details.value(Tr::tr("Message")).toString(),
                  QString::fromLatin1("message %1").arg(i));
-        QCOMPARE(details.value(model.tr("Location")).toString(),
+        QCOMPARE(details.value(Tr::tr("Location")).toString(),
                  QString::fromLatin1("somefile.js:%1").arg(i));
     }
 }
